                                                                    
import logging
from imagescanner import settings

class CustomStreamHandler(logging.StreamHandler, object):
    def emit(self, record):
        record.msg = '%s: %s' % (record.levelname, record.msg)
        super(CustomStreamHandler, self).emit(record)

def config_logger():
    handler = CustomStreamHandler()
    logging.root.setLevel(settings.LOGGING_LEVEL)
    logging.root.addHandler(handler)



import logging
from StringIO import StringIO

import Image
import twain
from imagescanner.backends import base

class ScannerManager(base.ScannerManager):
    
    def _refresh(self):
        self._devices = []
        src_manager = twain.SourceManager(0)
        devices = src_manager.GetSourceList()
        for dev in devices: 
            scanner_id = 'twain-%s' % len(self._devices)
            try:
                scanner = Scanner(scanner_id, dev)
                self._devices.append(scanner)
            except Exception, exc:
                # XXX: What should be here?
                # Debuging to try to find out
                logging.debug(exc)
        src_manager.destroy()

class Scanner(base.Scanner):  
    def __init__(self, scanner_id, source_name):
        self.id = scanner_id
        self._source_name = source_name

        self.name = None
        self.manufacturer = None
        self.description = None
        self._src_manager = None
        self._scanner = None
    
        self._open()
        self._get_identity()
        self._close()
    
    def _get_identity(self):
        identity = self._scanner.GetIdentity()
        self.name = identity.get('ProductName')
        self.manufacturer = identity.get('Manufacturer')
        self.description = None

    def _open(self):
        self._src_manager = twain.SourceManager(0)
        self._scanner = self._src_manager.OpenSource(self._source_name)
        self._scanner.SetCapability(twain.ICAP_YRESOLUTION, 
                                    twain.TWTY_FIX32, 200.0)

    def __repr__(self):
        return '<%s: %s - %s>' % (self.id, self.manufacturer, self.name)
    
    def scan(self, dpi=200):
        self._open()
        self._scanner.RequestAcquire(0, 0)
        info = self._scanner.GetImageInfo()
        if info:
            (handle, more_to_come) = self._scanner.XferImageNatively()
            str_image = twain.DIBToBMFile(handle)
            twain.GlobalHandleFree(handle)
            self._close()
            return Image.open(StringIO(str_image))

        self._close()
        return None
    
    def _close(self):
        self._scanner.destroy()
        self._src_manager.destroy()

    def status(self):
        pass



import sane

from imagescanner.backends import base 

class ScannerManager(base.ScannerManager):
 
    def _refresh(self):
        self._devices = []
        
        sane.init()
        devices = sane.get_devices()    
        for dev in devices:
            # Check if sane is able to open this device, if not just skip
            try:
                scanner = sane.open(dev[0])
                scanner.close()
            except:
                continue 
                                
            scanner_id = 'sane-%s' % len(self._devices)
            scanner = Scanner(scanner_id, dev[0], dev[1], dev[2], dev[3])
            self._devices.append(scanner)

        sane.exit()

class Scanner(base.Scanner):  
    def __init__(self, scanner_id, device, manufacturer, name, description):
        self.id = scanner_id
        self.manufacturer = manufacturer
        self.name = name
        self.description = description
        self._device = device

    def __repr__(self):
        return '<%s: %s - %s>' % (self.id, self.manufacturer, self.name)
    
    def scan(self, dpi=200):
        sane.init()
        scanner = sane.open(self._device)
        image = scanner.scan()
        scanner.close()
        sane.exit()

        return image




def scanner_serializer(device):
    return {
        'id': device.id,
        'name': device.name,
        'manufacturer': getattr(device, 'manufacturer', None),
        'description': getattr(device, 'description', None),
    }




import os
import Image

from imagescanner.backends import base 

class ScannerManager(base.ScannerManager):
 
    def _refresh(self):
        self._devices = []
        scanner = Scanner('test-0', "Pyscan", "Test Device")
        self._devices.append(scanner)

class Scanner(base.Scanner):  
    def __init__(self, scanner_id, manufacturer, name):
        self.id = scanner_id
        self.manufacturer = manufacturer
        self.name = name

    def __repr__(self):
        return "<%s: %s - %s>" % (self.id, self.manufacturer, self.name)
    
    def scan(self, dpi=200):
        imgpath = os.path.join(os.path.dirname(__file__), 'data', 'img1.tiff')
        return Image.open(imgpath)

    def status(self):
        pass



import cjson

from cStringIO import StringIO
from twisted.web import server
from twisted.web.xmlrpc import XMLRPC, Binary

from imagescanner import ImageScanner
from imagescanner.utils import scanner_serializer

class ScannerDevices(XMLRPC):

    def xmlrpc_list_scanners(self):
        devices = ImageScanner().list_scanners()
        serialized_devices = [scanner_serializer(device) for device in devices]
        return cjson.encode(serialized_devices)

    def xmlrpc_scan(self, device_id):
        image = ImageScanner().scan(device_id)
        if image is None:
            return None
        image_data = StringIO()
        image.save(image_data, 'tiff')
        image_data.seek(0) 
        return Binary(image_data.read())
    
def runserver(port):
    from twisted.internet import reactor
    
    root = ScannerDevices()
    reactor.listenTCP(port, server.Site(root))
    reactor.run()




from imagescanner.core._imagescanner import ImageScanner

# Configure the python logging using the custom handler
from imagescanner.utils.logger import config_logger
config_logger()
del config_logger



"""Abstract base classes"""

class ScannerManager(object):

    def __init__(self, **kwargs):
        self._devices = []
    
    def _refresh(self):
        """Look for new scanner devices"""
        raise NotImplementedError

    def get_scanner(self, scanner_id):
        """Return a scanner with the given ID or None if not found"""
        devices = self.list_scanners()

        for dev in devices:
            if dev.id == scanner_id:
                return dev
        return None

    def list_scanners(self):
        """Return a list with all the available devices"""
        self._refresh()
        return self._devices      

class Scanner(object):
    """Abstract Scanner class"""

    def scan(self, dpi=200):
        """Scan a new image using the given DPI and returns a PIL object"""
        raise NotImplementedError

    def status(self):
        """Get device status"""
        # TODO: Define standard status
        raise NotImplementedError



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='imagescanner',
      version='0.8',
      description='Multi-platform Python library to access scanner devices.',
      author='Sergio Campos',
      author_email='seocam@seocam.net',
      url='http://code.google.com/p/imagescanner/',
      packages=find_packages(),
      package_data={'': ['*.tiff']},
      install_requires=[],
     )




import xmlrpclib
import socket
import logging
from cStringIO import StringIO

import Image
import cjson
from imagescanner import settings
from imagescanner.backends import base

class ScannerManager(base.ScannerManager):

    def __init__(self, **kwargs):
        super(ScannerManager, self).__init__(**kwargs)
        remote_hosts = kwargs.get('remote_host', tuple())       
 
        self._proxies = []
        for host in remote_hosts:
            proxy = xmlrpclib.ServerProxy("http://%s/" % host, allow_none=True)
            self._proxies.append(proxy)

    def _refresh(self):
        logging.debug('Reloading remote device information')
        self._devices = []
        for proxy in self._proxies:
            # TODO: Redo it without accessing protected members
            remote_host = proxy._ServerProxy__host
            try:
                response = proxy.list_scanners()
            except socket.error:
                logging.error('Connection refused when trying to list '
                              'scanners in %s [skiping]', remote_host)
                continue
            logging.debug('JSON answer from %s: %s', remote_host, response)
            
            scanner_list = cjson.decode(response)
            for scanner_info in scanner_list:
                scanner = Scanner(proxy, **scanner_info)
                self._devices.append(scanner)
        logging.debug('Remote devices loaded: %s', self._devices)
 
class Scanner(base.Scanner):

    def __init__(self, **kwargs):
        # Different hosts can have the same id, so the host need to be part of
        #   the scanner id
        scanner_id = kwargs.get('scanner_id', None)
        proxy = kwargs.get('proxy', None)
        remote_host = proxy._ServerProxy__host

        self.id = "%s/%s" % (remote_host, scanner_id)
        self._remote_id = scanner_id
        self.name = kwargs.get('name', None)
        self.manufacturer = kwargs.get('manufacturer', None)
        self.description = kwargs.get('description', None)
        self._proxy = proxy

    def scan(self, dpi=200):
        binary = self._proxy.scan(self._remote_id)
        if binary is None:
            return None

        image_data = StringIO()
        image_data.write(binary.data)
        image_data.seek(0)
        return Image.open(image_data)
    
    def __repr__(self):
        return '<%s: %s - %s>' % (self.id, self.manufacturer, self.name)

    def status(self):
        pass



import os
import logging

from imagescanner import settings
from imagescanner.utils.importlib import import_module

POSIX_BACKEND = 'imagescanner.backends.sane'
NT_BACKEND = 'imagescanner.backends.twain'
NETWORK_BACKEND = 'imagescanner.backends.net'
TEST_BACKEND = 'imagescanner.backends.test'

BACKENDS = getattr(settings, 'BACKENDS', None)

class ImageScanner(object):
    def __init__(self, **kwargs):
        self.managers = []
        backends = []

        # Enable default backend for each system
        if os.name == 'posix':
            logging.debug('Posix backend enabled (%s)', POSIX_BACKEND)
            backends.append(POSIX_BACKEND)
        elif os.name == 'nt':
            logging.debug('NT backend enabled (%s)', NT_BACKEND)
            backends.append(NT_BACKEND)
        
        # Check if we should load the test backend
        if hasattr(settings, 'ENABLE_TEST_BACKEND'):
            if settings.ENABLE_TEST_BACKEND:
                logging.debug('Test backend enabled (%s)', TEST_BACKEND)
                backends.append(TEST_BACKEND)
        
        # If trying to scan using a remote host enable network backend
        if kwargs.get('remote_hosts'):
            logging.debug('Network backend enabled (%s)', NETWORK_BACKEND)
            backends.append(NETWORK_BACKEND)

        # Check if the user has own backends defined
        if BACKENDS is not None:
            logging.debug('Adding user backends (%s)', BACKENDS)
            backends.extend(BACKENDS)
        
        for backend in backends:    
            try:
                logging.debug('Importing %s', backend)
                backend_module = import_module(backend)
            except Exception, exc:
                logging.warning('Error importing %s [skiping]', backend)
                logging.warning(exc)
                # Could not import, so go to the next backend 
                continue

            try:
                manager = backend_module.ScannerManager(**kwargs)
            except AttributeError:
                logging.error('Backend %s does not implement '
                              'ScannerManager class [skiping]', backend)
                # Backend not implement the required API, so skip it
                continue 
        
            self.managers.append(manager)

    def list_scanners(self):
        """List all devices for all the backends available"""
        logging.debug('Looking for all scanner devices') 
        scanners = []
        for manager in self.managers:
            scanners.extend(manager.list_scanners())
       
        logging.debug('Found scanners: %s', scanners) 
        return scanners

    def get_scanner(self, scanner_id):
        """Get the device with the given ID"""
        logging.debug('Looking for scanner with id: %s', scanner_id)
        scanner = None
        for manager in self.managers:
            scanner = manager.get_scanner(scanner_id)
            if scanner is not None:
                logging.debug('Scanner %s found', scanner_id)
                return scanner
        logging.debug('Scanner %s not found', scanner_id)

    def scan(self, scanner_id, **kwargs):
        """Shortcut for scanning without get the device"""

        logging.debug('Trying to scan using: %s and %s', scanner_id, kwargs)
        scanner = self.get_scanner(scanner_id)
        if scanner is not None:
            return scanner.scan(**kwargs)
       
        logging.info('Scan failed. Scanner %s not found', scanner_id)
